#ifndef GENERIC_FI_H
#define GENERIC_FI_H

/* Compilers may use another function within the assert macro to exit the
 * program. QAC thus does not see an "exit" function and assumes the
 * function returns, which it does not.
 * The below pragma tells QAC the compiler supplied function does not
 * return.
 */

/* examples - GCC, ICC, BCC 3.1, BCC 5.5, CYGWIN gcc, Watcom 1.6 */
#pragma PRQA_NO_RETURN __assert_fail
#pragma PRQA_NO_RETURN __assert_perror_fail
#pragma PRQA_NO_RETURN __assert

/* Depending on the code that the compiler maker uses within the assert macro,
 * QAC may issue warnings. It is also possible that when the NDEBUG macro is used
 * to suppress the assertion code, QAC may issue a "no side-effects" warning.
 * To eliminate these, we turn off all warnings for the assert macro.
 */
#pragma PRQA_MACRO_MESSAGES_OFF "assert"
#pragma PRQA_MACRO_MESSAGES_OFF "errno"


/* Any other items that may be required in a "force-include" file should
 * be placed below.
 */

/* to avoid problems with GCC using builtin functions for va_start
 * and QAC issuing unset variable warnings.
 */
typedef void * __builtin_va_list;
#define __builtin_va_start(ap, parmN)	((void)((ap)= &(parmN)))
#define __builtin_va_arg(ap, type)	(*(type*)(ap))
#define __builtin_va_end(ap)		((void)((ap)=0))
#define __builtin_va_copy(d,s)
#pragma PRQA_MACRO_MESSAGES_OFF "va_end"
#pragma PRQA_MACRO_MESSAGES_OFF "__va_copy"

#define  __builtin_offsetof(type, member) ((size_t) &((type *)0)->member)

/* GCC builtin function equivalents that require prototypes
 * for QAC.
 */

extern long __builtin_expect (long exp, long c);
extern void *__builtin_memcpy(void *, const void *, __SIZE_TYPE__);
extern void *__builtin_mempcpy(void *, const void *, __SIZE_TYPE__);
extern void *__builtin_memset(void *, int, __SIZE_TYPE__);
extern int __builtin_memcmp(const void *, const void *, __SIZE_TYPE__);
extern char *__builtin_strcat(char *, const char *);
extern char *__builtin_strncat(char *, const char *, __SIZE_TYPE__);
extern int __builtin_strcmp(const char *, const char *);
extern char *__builtin_strchr(const char *, int);
extern char *__builtin_strcpy(char *, const char *);
extern char *__builtin_strncpy(char *, const char *, __SIZE_TYPE__);
extern __SIZE_TYPE__ __builtin_strspn(const char *, const char *);
extern __SIZE_TYPE__ __builtin_strcspn(const char *, const char *);
extern char * __builtin_strpbrk(const char *, const char *);
extern __SIZE_TYPE__ __builtin_strlen(const char *);
extern int __builtin_clz(int);
extern int __builtin_clzl(long);
extern int __builtin_clzll(long long);
extern int __builtin_ctz(int);
extern int __builtin_ctzl(long);
extern int __builtin_ctzll(long long);
extern int __builtin_ffs(int);
extern int __builtin_ffsl(long);
extern int __builtin_ffsll(long long);
extern int __builtin_popcount(unsigned int);
extern int __builtin_popcountl(unsigned long);
extern int __builtin_popcountll(unsigned long long);
extern void *__builtin_return_address(unsigned int);
extern void *__builtin_extract_return_addr(void *);
extern void *__builtin_frame_address(unsigned int);
extern void __builtin_trap(void);
extern void *__builtin_alloca(__SIZE_TYPE__);
extern void __builtin_prefetch (const void *, ...);
extern long __builtin_alpha_extbl(long, long);
extern long __builtin_alpha_extwl(long, long);
extern long __builtin_alpha_insbl(long, long);
extern long __builtin_alpha_inswl(long, long);
extern long __builtin_alpha_insql(long, long);
extern long __builtin_alpha_inslh(long, long);
extern long __builtin_alpha_cmpbge(long, long);
extern long __builtin_labs(long);
extern int __builtin_isgreater(float, float);
extern int __builtin_isgreaterequal(float, float);
extern int __builtin_isless(float, float);
extern int __builtin_islessequal(float, float);
extern int __builtin_islessgreater(float, float);
extern int __builtin_isunordered(float, float);
extern __SIZE_TYPE__ __builtin_object_size(void *, int);
extern void * __builtin___memcpy_chk(void *, const void *, __SIZE_TYPE__, __SIZE_TYPE__);
extern void * __builtin___memmove_chk(void *, const void *, __SIZE_TYPE__, __SIZE_TYPE__);
extern void * __builtin___mempcpy_chk(void *, const void *, __SIZE_TYPE__, __SIZE_TYPE__);
extern void * __builtin___memset_chk(void *, int, __SIZE_TYPE__, __SIZE_TYPE__);
extern int __builtin___sprintf_chk(char *, int, __SIZE_TYPE__, const char *, ...);
extern int __builtin___snprintf_chk(char *, __SIZE_TYPE__, int , __SIZE_TYPE__, const char *, ...);
extern char * __builtin___stpcpy_chk(char *, const char *, __SIZE_TYPE__);
extern char * __builtin___strcat_chk(char *, const char *, __SIZE_TYPE__);
extern char * __builtin___strcpy_chk(char *, const char *, __SIZE_TYPE__);
extern char * __builtin___strncat_chk(char *, const char *, __SIZE_TYPE__, __SIZE_TYPE__);
extern char * __builtin___strncpy_chk(char *, const char *, __SIZE_TYPE__, __SIZE_TYPE__);

#else
#error "Multiple include"
#endif  /* ifndef GENERIC_FI_H */
